@@ -8,7 +8,7 @@
 #[doc(hidden)]
 #[macro_export]
 macro_rules! em_if {
-    ( $i:ident, $p:pat, $t:expr, $f:expr ) => {
+    ( $i:expr, $p:pat, $t:expr, $f:expr ) => {
         if let $p = $i {
             $t
         } else {
@@ -20,7 +20,7 @@ macro_rules! em_if {
 #[doc(hidden)]
 #[macro_export]
 macro_rules! get {
-    ( $i:ident, $p:path, $pp:tt, $r:expr ) => {
+    ( $i:expr, $p:path, $pp:tt, $r:expr ) => {
         $crate::em_if!($i, $p$pp, Some($r), None)
     };
 }
@@ -28,7 +28,7 @@ macro_rules! get {
 #[doc(hidden)]
 #[macro_export]
 macro_rules! is {
-    ( $i:ident, $p:path, $pp:tt, $r:expr ) => {
+    ( $i:expr, $p:path, $pp:tt, $r:expr ) => {
         $crate::em_if!($i, $p$pp, true, false)
     };
 }
@@ -71,13 +71,16 @@ macro_rules! and_then {
 /// // Get Inner Value
 ///
 /// assert_eq!(em!(a.get Enum::A), Some(1));
+/// assert_eq!(em!(Enum::A(1) => get Enum::A), Some(1));
 /// assert_eq!(em!(b.get Enum::B[i, j]), Some((2, 3)));
 /// assert_eq!(em!(c.get Enum::C{x, y}), Some((4, 5)));
 /// assert_eq!(em!(c.get Enum::C{y, x}), Some((5, 4)));
+/// assert_eq!(em!(c.get Enum::C{x, ..}), Some((4,)));
 ///
 /// // Variants Detect
 ///
 /// assert_eq!(em!(a.is Enum::A), true);
+/// assert_eq!(em!(Enum::A(1) => is Enum::A), true);
 ///
 /// assert_eq!(em!(b.is Enum::C{x, y}), false);
 /// // or
@@ -121,11 +124,20 @@ macro_rules! em {
     ( $i:ident.$m:ident $p:path[$($x:ident),+] $(> $($a:expr),+)? ) => {
         $crate::$m!($i, $p, ($($x,)+), ($($x,)+) $(, $($a),+)?)
     };
-    ( $i:ident.$m:ident $p:path{$(..)?} $(> $($a:expr),+)? ) => {
-        $crate::$m!($i, $p, { .. }, () $(, $($a),+)?)
+    ( $i:ident.$m:ident $p:path{$($($x:ident),+$(,)?)? $(..)?} $(> $($a:expr),+)? ) => {
+        $crate::$m!($i, $p, { $($($x,)+)? .. }, ($($($x,)+)?) $(, $($a),+)?)
     };
-    ( $i:ident.$m:ident $p:path{$($x:ident),+} $(> $($a:expr),+)? ) => {
-        $crate::$m!($i, $p, {$($x,)+}, ($($x,)+) $(, $($a),+)?)
+    ( $i:expr => $m:ident $p:path $(> $($a:expr),+)? ) => {
+        $crate::$m!($i, $p, (_x), _x $(, $($a),+)?)
+    };
+    ( $i:expr => $m:ident $p:path[$(..)?] $(> $($a:expr),+)? ) => {
+        $crate::$m!($i, $p, (..), () $(, $($a),+)?)
+    };
+    ( $i:expr => $m:ident $p:path[$($x:ident),+] $(> $($a:expr),+)? ) => {
+        $crate::$m!($i, $p, ($($x,)+), ($($x,)+) $(, $($a),+)?)
+    };
+    ( $i:expr => $m:ident $p:path{$($($x:ident),+$(,)?)? $(..)?} $(> $($a:expr),+)? ) => {
+        $crate::$m!($i, $p, { $($($x,)+)? .. }, ($($($x,)+)?) $(, $($a),+)?)
     };
 }
 
@@ -149,8 +161,10 @@ mod tests {
     fn get() {
         let (a, b, c) = init();
         assert_eq!(em!(a.get Enum::A), Some(1));
+        assert_eq!(em!(Enum::A(1) => get Enum::A), Some(1));
         assert_eq!(em!(b.get Enum::B[i, j]), Some((2, 3)));
         assert_eq!(em!(c.get Enum::C{x, y}), Some((4, 5)));
+        assert_eq!(em!(c.get Enum::C{x, ..}), Some((4,)));
     }
 
     #[test]
